package com.ozs.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ozs.common.annotation.Excel;
import com.ozs.common.core.domain.BaseEntity;
import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 报警信息表
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@ApiModel("报警信息表")
public class MsgAlarm extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 报警唯一标识
     */
    @Excel(name = "报警唯一标识")
    private String alarmId;

    /**
     * 报警时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "报警时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date alarmTime;

    /**
     * 报警线路
     */
    private String railwayCode;

    /**
     * 报警里程位置
     */
    private Integer alarmMile;

    /**
     * 报警内容
     */
    @Excel(name = "报警内容")
    private String content;

    /**
     * 报警级别
     */
//    @Excel(name = "报警级别")
    private Integer alarmLevel;

    /**
     * 报警置信度（百分比）
     */
    private Integer alarmConfidence;

    /**
     * 报警行别(1上行2下行)
     */
    private Integer lineDir;

    /**
     * 报警病害类型（1泥石流2异物侵限 3断轨监测）
     */
    private Integer alarmType;
    @TableField(exist = false)
    private String alarmTypeStr;

    /**
     * 框定范围
     */
    private String alarmAttr;

    /**
     * 解除人
     */
    private String releasedBy;

    /**
     * 解除时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "解除时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date releasedTime;

    /**
     * 报警是否解除 1已解除2未解除
     */
    private Integer isRelease;

    /**
     * 创建人
     */
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 修改人
     */
    private String updateBy;

    /**
     * 修改时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    /**
     * 备注
     */
    private String remark;

    /**
     * 解除内容
     */
    @Excel(name = "解除内容")
    private String releaseContent;

    /**
     * 线路名称
     */
    @TableField(exist = false)
    @Excel(name = "报警线路")
    private String railwayName;

    /**
     * 监控范围开始里程位置
     */
    @TableField(exist = false)
    private String beginMile;

    /**
     * 监控范围结束里程位置
     */
    @TableField(exist = false)
    private String endMile;

    /**
     * 报警里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "报警里程位置")
    private String alarmMiles;

    /**
     * 报警次数
     */
    @TableField(exist = false)
    private Integer alarmCount;

    /**
     * 报警图片地址网络路径
     */
    @TableField(exist = false)
    private List<String> imageUrls;

    /**
     * 解除人真实姓名
     */
    @TableField(exist = false)
    @Excel(name = "解除人")
    private String releasedByName;

    /**
     * 状态：1未读；2已读
     */
    @TableField(exist = false)
    private Integer status;

    /**
     * 推送消息ID
     */
    @TableField(exist = false)
    private String msgId;

    /**
     * 报警行别字符串(1上行2下行)
     */
    @TableField(exist = false)
    @Excel(name = "报警行别")
    private String lineDirName;

    /**
     * 报警病害类型（目前只有1泥石流，后续可扩展使用）
     */
    @TableField(exist = false)
    @Excel(name = "报警病害类型")
    private String alarmTypeName;

    /**
     * 报警是否解除 1已解除2未解除
     */
    @Excel(name = "报警是否解除")
    @TableField(exist = false)
    private String  isReleaseName;

    /**
     * 报警视频URL
     */
    private String releasedUrl;

    /**
     * 来源
     */
    private String source;

    /**
     * 来源名称
     */
    @TableField(exist = false)
    private String sourceName;

    /**
     * 车次
     */
    @TableField(exist = false)
    @Excel(name = "车次")
    private  String trainNumName;


    /**
     * 机车号
     */
    @Excel(name = "车号")
    @TableField(exist = false)
    private String vehicleCode;


    /**
     * 终端名称
     */
    @TableField(exist = false)
    @Excel(name = "终端名称")
    private String terminalName;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "推送时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @TableField(exist = false)
    private Date createTimes;
}
