package com.ozs.entity.vo;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ozs.common.annotation.Excel;
import com.ozs.common.core.domain.BaseEntity;
import com.ozs.entity.BaseRailwayManagement;
import com.ozs.entity.MsgAlarmExt;
import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 报警信息临时表
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@ApiModel("报警信息临时表")
public class MsgAlarmTemporary  implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 报警唯一标识
     */
    private String alarmId;

    /**
     * 报警时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date alarmTime;

    /**
     * 报警线路
     */
    private String alarmRailway;

    /**
     * 报警里程位置
     */
    private Integer alarmMile;

    /**
     * 报警内容
     */
    @Excel(name = "报警内容")
    private String content;

    /**
     * 报警级别
     */
    private Integer alarmLevel;

    /**
     * 报警置信度（百分比）
     */
    private Integer alarmConfidence;

    /**
     * 报警行别(1上行2下行)
     */
    private Integer lineDir;

    /**
     * 报警病害类型（1泥石流2异物侵限 3断轨监测）
     */
    private Integer alarmType;
    @TableField(exist = false)
    @Excel(name = "报警病害类型")
    private String alarmTypeStr;

    /**
     * 框定范围
     */
    private String alarmAttr;

    private String fileUrl;
    /**
     * 模拟状态 1未使用2已使用
     */
    private Integer stare;
    /**
     * 模拟状态 1未使用2已使用
     */
    @TableField(exist = false)
    private String stareName;
    
    /**
     * 线路名称
     */
    @TableField(exist = false)
    @Excel(name = "报警线路")
    private String railwayName;
    

    /**
     * 报警里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "报警里程位置")
    private String alarmMiles;
    

    /**
     * 报警行别字符串(1上行2下行)
     */
    @TableField(exist = false)
    @Excel(name = "报警行别")
    private String lineDirName;

    /**
     * 报警病害类型（目前只有1泥石流，后续可扩展使用）
     */
    @TableField(exist = false)
    @Excel(name = "报警病害类型")
    private String alarmTypeName;


    /**
     * 来源
     */
    private String source;

    private  String trainNum;

}
